;; four models are implemented from Model 1 through Model 4. 
;; you can choose one of them by using the slider.
;; both Model 2 and Model 3 operate only if the tag length is at least 1.
;; if you want to explore the dynamics of cooperation in structured populations 
;; consisting of unconditional cooperators and defectors, you should choose Model 1.
;; the slider of mu_factor is relevant to Model 4.

turtles-own [      ;; agent's traits
  cooperate?       
  strategy-list    ;; a list of each agent's strategies per generation
  score            
  score_list       ;; a list of scores each agent earns from interactions with her neighbors 
  tag-list
  tolerance       
  dissimilarity_list         ;; a list of hamming distances toward her neighbors
  average_hamming_distance   ;; the average of perceived social distances toward her neighbors 
  neighbor_list
  neighborhood
  immutable-list     ;; if 1 at a certain dimension of the tag-list, tags in that dimension do not change.
                          ;; for all agents in Model 2, 1 is assigned at the first dimension. 
]

globals [        ;; global variables
  go?
  proportion_cooperation_list
  homophily_list_coop             ;; local measure of tolerance association (homophily)
  homophily_coop
  homophily_list_distance         ;; local measure of tag association (homophily)
  homophily_distance
]

to setup
  ca
  ask patches [ 
    create-agents 
  ]
  ask turtles [ 
    set neighborhood turtles-on neighbors      ;; the Moore neighborhood
    set neighbor_list [ who ] of neighborhood 
    similarity-perception
    establish-color-setup
  ]
  update-plot
end

to create-agents
  if which_model? = 1 or which_model? = 4 [     ;; Model 1 or 4
    sprout 1 [
      set tag-list [ ]
      let i 0
      while [ i < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list    ;; uniform distribution
        set i i + 1
      ]     
      let l num-of-tag-item
      set tolerance random ( l + 2 )           ;; uniform distribution
      set dissimilarity_list [ ]
      set score_list [ ]
      set strategy-list [ ]
      set shape "circle"    
    ]
  ]
  if which_model? = 2 [                        ;; Model 2 named "a caste society"
    sprout 1 [
      set tag-list [ ]
      set immutable-list [ ]
      let i 0
      while [ i < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list    ;; uniform distribution
        set immutable-list fput 0 immutable-list
        set i i + 1
      ]     
      set immutable-list replace-item 0 immutable-list 1
      let l num-of-tag-item
      set tolerance random ( l + 2 )           ;; uniform distribution
      set dissimilarity_list [ ]
      set score_list [ ]
      set strategy-list [ ]
      set shape "circle"    
    ]
  ]  
  if which_model? = 3 [                       ;; Model 3 named "a modern society"
    sprout 1 [
      set tag-list [ ]
      set immutable-list [ ]
      let i 0
      while [ i < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list    ;; uniform distribution
        set immutable-list fput 0 immutable-list
        set i i + 1
      ]     
      set immutable-list replace-item ( random num-of-tag-item ) immutable-list 1
      let l num-of-tag-item
      set tolerance random ( l + 2 )           ;; uniform distribution
      set dissimilarity_list [ ]
      set score_list [ ]
      set strategy-list [ ]
      set shape "circle"    
    ]
  ]    
end

to establish-color-setup 
  set color 1 + tolerance     ;; the darker, the higher tolerance; the lighter, the lower tolerance  
end

to go
  set go? true 
  ask turtles [
    similarity-perception
  ]
  ask turtles [
    interaction
  ]
  ask turtles [
    payoff-calculation
  ]
  ask turtles [
    tag-tolerance-update  
  ]
  tick
  ask turtles [
    select present_what
  ]
  update-plot
end

to similarity-perception
  set dissimilarity_list [ ]
  let i 0
  while [ i < 8 ]
  [ 
    let b length filter [ ? = true ] ( map [ ?1 = 1 and ?2 = 0 ] [ tag-list ] of self [ tag-list ] of turtle item i neighbor_list ) 
    let c length filter [ ? = true ] ( map [ ?1 = 0 and ?2 = 1 ] [ tag-list ] of self [ tag-list ] of turtle item i neighbor_list )
    set dissimilarity_list lput ( b + c ) dissimilarity_list
    set i i + 1
  ]    
  set average_hamming_distance mean dissimilarity_list
end

to interaction
  set score_list [ ]
  set strategy-list [ ]
  let i 0
  while [ i < 8 ] [
    let partner turtle item i neighbor_list
    let ego self
    let p position [ who ] of ego [ neighbor_list ] of partner
    ask partner [
      ifelse [ tolerance ] of partner > item p [ dissimilarity_list ] of partner [
        set [ cooperate? ] of partner true
      ] [
        set [ cooperate? ] of partner false
      ]
    ]
    ifelse [ tolerance ] of ego > item i [ dissimilarity_list ] of ego [ 
      set [ cooperate? ] of ego true
      set strategy-list lput 1 strategy-list
      ifelse [ cooperate? ] of partner = true [
        set score_list lput ( b/c - 1 ) score_list      ;; equivalent to the reward for mutual coopreation (R)
      ] [
        set score_list lput ( - 1 ) score_list          ;; equivalent to the sucker's payoff (S)
      ] 
    ] [
      set [ cooperate? ] of ego false
      set strategy-list lput 0 strategy-list
      ifelse [ cooperate? ] of partner = true [
        set score_list lput b/c score_list              ;; equivalent to the temptation to defect (T) 
      ] [
        set score_list lput 0 score_list                ;; equivalent to the punishment for mutual defection (P)
      ]     
    ]  
    set i i + 1
  ]
end

to payoff-calculation
  set score sum score_list 
end

to tag-tolerance-update  
  if which_model? = 1 [                                 ;; Model 1
    let reference one-of turtles-on neighbors
    let what-to-compare [ score ] of reference
    let self-score [ score ] of self
    ifelse mutation-rate > random-float 1.0 [           ;; if mutation occurs
      set tag-list [ ]
      let j 0
      while [ j < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list
        set j j + 1
      ] 
      set tolerance random ( num-of-tag-item + 2 )
    ] [                                                 ;; otherwise, payoff-based learning
      if what-to-compare > self-score [
        set tag-list [ ]
        set tag-list [ tag-list ] of reference
        set tolerance [ tolerance ] of reference
      ]
    ]
  ]
  if which_model? = 2 [                                 ;; Model 2
    let reference one-of turtles-on neighbors
    let what-to-compare [ score ] of reference
    let self-score [ score ] of self
    ifelse mutation-rate > random-float 1.0 [           ;; if mutation occurs
      let a item 0 tag-list
      set tag-list [ ]
      let j 0
      while [ j < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list
        set j j + 1
      ] 
      set tag-list replace-item 0 tag-list a
      set tolerance random ( num-of-tag-item + 2 )
    ] [                                                 ;; otherwise, payoff-based learning
      if what-to-compare > self-score [
        let a item 0 tag-list
        set tag-list [ ]
        set tag-list [ tag-list ] of reference
        set tag-list replace-item 0 tag-list a
        set tolerance [ tolerance ] of reference
      ]
    ]
  ]  
  if which_model? = 3 [                                 ;; Model 3
    let reference one-of turtles-on neighbors
    let what-to-compare [ score ] of reference
    let self-score [ score ] of self
    ifelse mutation-rate > random-float 1.0 [           ;; if mutation occurs
      let p position 1 immutable-list
      let a item p tag-list
      set tag-list [ ]
      let j 0
      while [ j < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list
        set j j + 1
      ] 
      set tag-list replace-item p tag-list a
      set tolerance random ( num-of-tag-item + 2 )
    ] [                                                 ;; otherwise, payoff-based learning
      if what-to-compare > self-score [
        let p position 1 immutable-list
        let a item p tag-list
        set tag-list [ ]
        set tag-list [ tag-list ] of reference
        set tag-list replace-item p tag-list a
        set tolerance [ tolerance ] of reference
      ]
    ]
  ]    
  if which_model? = 4 [                                ;; Model 4
    let reference one-of turtles-on neighbors
    let what-to-compare [ score ] of reference
    let self-score [ score ] of self
    let a random-float 1.0
    ifelse mutation-rate * mu_factor > a [             ;; if mutation occurs (tag change)
      set tag-list [ ]
      let j 0
      while [ j < num-of-tag-item ]
      [ set tag-list fput random 2 tag-list
        set j j + 1
      ] 
    ] [                                               ;; otherwise, payoff-based learning (tag change)
      if what-to-compare > self-score [              
        set tag-list [ ]
        set tag-list [ tag-list ] of reference
      ]      
    ]
    ifelse mutation-rate > a [                        ;; if mutation occurs (tolerance change)
      set tolerance random ( num-of-tag-item + 2 )
    ] [                                               ;; otherwise, payoff-based learning (tolerance change)
      if what-to-compare > self-score [
        set tolerance [ tolerance ] of reference
      ]
    ]
  ]  
end

to select [ present ]
  if ( present = "strategy" ) [
    set color 41 + sum strategy-list                  ;; Min (sum of strategy-list) = 0 and Max (sum of strategy-list) = 8
  ]
  if ( present = "av-hamming-d" ) [ 
    set color 127 - 10 * average_hamming_distance     ;; the darker, the more heterogeneous
  ]
end

to update-plot
  set-current-plot "tolerance distribution"
  plot-pen-reset
  set-plot-x-range 0 num-of-tag-item + 2
  set-plot-y-range 0 count turtles
  set-histogram-num-bars ( num-of-tag-item + 2 )   
  histogram [ tolerance ] of turtles
  
  set-current-plot "average dissimilarity distribution"
  plot-pen-reset
  ifelse num-of-tag-item = 0 [
    set-plot-x-range 0 1
  ] [
    set-plot-x-range 0 num-of-tag-item 
  ]
  set-plot-y-range 0 count turtles
  set-histogram-num-bars 10
  histogram [ average_hamming_distance ] of turtles  
    
  ifelse count turtles with [ tolerance >= 1 ] > 0 [         ;; local measure of tolerance homophily
    let i 0
    set homophily_list_coop [ ]
    while [ i < count turtles ] [
      ask turtle i [
        let z [ tolerance ] of self
        if z >= 1 [
          set homophily_list_coop lput ( count neighborhood with [ tolerance = z ] / count [ neighborhood ] of self ) homophily_list_coop 
        ]
      ]
      set i i + 1
    ]
    set homophily_coop sum homophily_list_coop / count turtles with [ tolerance >= 1 ]     ;; we exclude uncondiitonal defectors (T = 0)
  ] [ 
    set homophily_coop 0
  ]
 
  let i 0                                                     ;; local measure of tag homophily
  set homophily_list_distance [ ]
  while [ i < count turtles ] [
    ask turtle i [
      let z [ average_hamming_distance ] of self
      set homophily_list_distance lput ( count neighborhood with [ average_hamming_distance = z ] / count [ neighborhood ] of self ) homophily_list_distance 
    ]
    set i i + 1
  ]
  set homophily_distance sum homophily_list_distance / count turtles     
    
  set-current-plot "local homophily"
  set-current-plot-pen "tol"
  plotxy ticks homophily_coop
  set-current-plot-pen "tag"
  plotxy ticks homophily_distance       
  
  if go? = true [
    set proportion_cooperation_list [ ]
    let j 0 
    while [ j < count turtles ]
    [
      set proportion_cooperation_list lput ( ( sum [ strategy-list ] of turtle j ) / 8 ) proportion_cooperation_list
      set j j + 1
    ] 
    set-current-plot "proportion of cooperation"
    plot ( sum proportion_cooperation_list ) / count turtles 
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
169
12
405
269
-1
-1
22.6
1
10
1
1
1
0
1
1
1
0
9
0
9
1
1
1
ticks

CC-WINDOW
5
422
677
517
Command Center
0

BUTTON
38
53
102
86
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
103
53
167
86
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
38
88
169
121
b/c
b/c
2
10
4
0.1
1
NIL
HORIZONTAL

SLIDER
38
158
168
191
num-of-tag-item
num-of-tag-item
0
10
5
1
1
NIL
HORIZONTAL

PLOT
406
135
668
271
tolerance distribution
tolerance level
# of agents
0.0
11.0
0.0
100.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
38
123
168
156
mutation-rate
mutation-rate
0
1
0.01
0.001
1
NIL
HORIZONTAL

PLOT
406
10
666
134
proportion of cooperation
time
p_coop
0.0
50.0
0.0
1.0
true
false
PENS
"default" 1.0 0 -10022847 true

CHOOSER
37
228
167
273
present_what
present_what
"strategy" "av-hamming-d"
0

PLOT
406
271
668
408
average dissimilarity distribution
average hamming distance
# of agents
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

MONITOR
36
327
166
376
p_coop
mean proportion_cooperation_list
4
1
12

PLOT
170
272
405
408
local homophily
time
NIL
0.0
50.0
0.0
1.0
true
true
PENS
"tol" 1.0 2 -4079321 true
"tag" 1.0 0 -4757638 true

MONITOR
36
276
167
325
global_av_hd
mean [ average_hamming_distance ] of turtles
17
1
12

SLIDER
46
11
160
44
which_model?
which_model?
1
4
2
1
1
NIL
HORIZONTAL

SLIDER
37
193
167
226
mu_factor
mu_factor
1
30
1
1
1
NIL
HORIZONTAL

@#$#@#$#@
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
